<?php

// define emails which should be sent TO seller
function bt_wc_emails_to_seller() {
	return [
		'new_order',
		'cancelled_order',
		'failed_order',
		'low_stock',
		'no_stock',
	];
}

// define emails which should be sent FROM seller
function bt_wc_emails_from_seller() {
	return [
		'customer_processing_order',
		'customer_completed_order',
		'customer_ready_pickup_order',
		'customer_completed_picked_order',
		'customer_completed_shipped_order',
		'customer_invoice',
		'customer_note',
		'customer_on_hold_order',
		'customer_refunded_order',
	];
}


/**
 * Hook into email headers (for any woocommerce email) when using Sellers to:
 * - Emails TO seller: set site admin as bcc
 * - Emails FROM seller: set seller as reply-to
 *
 * @param $headers string
 * @param $email_id string like 'customer_processing_order'
 * @param $order \WC_Order or some admin override version of it
 *
 * @return string the headers, modified as needed
 */
function bt_wc_email_headers( $headers, $email_id, $order ) {

	// things to do if we are using sellers
	if ( bt_use_sellers() ) {

		// if an email TO seller (e.g. new_order)
		if ( in_array( $email_id, bt_wc_emails_to_seller() ) ) {
			// add site admin as bcc
			$headers .= "Bcc: " . get_bloginfo( 'name' ) . " <" . get_bloginfo( 'admin_email' ) . ">" . "\r\n";
		}

		// if an email that should be FROM seller
		if ( in_array( $email_id, bt_wc_emails_from_seller() ) ) {
			
			$seller_data = bt_get_order_seller_data( $order->get_id() );

			// break up the lines to find what we need to update
			$header_lines = explode( "\r\n", $headers );

			foreach ( $header_lines as $key => $line ) {
				// add reply-to seller for customer emails
				if ( strpos( $line, 'Reply-to' ) !== false ) {
					// replace this line with a reply-to for the seller
					$header_lines[$key] = "Reply-to: " . $seller_data->post->post_title . " <" . $seller_data->user_email . ">";
				}
			}

			// reassemble
			$headers = implode( "\r\n", $header_lines );
		}
	}

	//error_log( 'WOO EMAIL HEADERS FN. headers after process = ' . PHP_EOL . $headers /*. ' order = ' . print_r($order,1)*/ );

	return $headers;
}
add_filter( 'woocommerce_email_headers', 'bt_wc_email_headers', 9998, 3 );


/**
 * Hook into "from" email address (for any woocommerce email) to:
 *
 * - if sellers, set from to the seller's email
 *      ?? maybe we shouldn't, for deliverability! We'll try without this, and just reply-to, first.
 *
 * @param $from_email string plain email address
 * @param $wc_email \WC_Email
 * @param $wp_default_server_email string not the order data unfortunately...
 *
 * @return string
 * /
function bt_wc_email_from_address( $from_email, $wc_email, $wp_default_server_email ) {

	// things to do if we are using sellers
	if ( bt_use_sellers() ) {

		if ( in_array( $wc_email->id, bt_wc_emails_from_seller() ) ) {
			//error_log( 'WOO EMAIL FROM ADDRESS FN. FILTERING email = ' . $wc_email->id );

			// get seller email and change the recipient to that so long as it exists (it always should)
			$seller_email = bt_get_order_seller_email( $wc_email->object->get_id() );

			if ( ! empty( $seller_email ) ) {
				$from_email = $seller_email;
			}
		}

	}

	//error_log( 'WOO EMAIL FROM ADDRESS FN. from_email result = ' . $from_email );

	return $from_email;
}
add_filter( 'woocommerce_email_from_address', 'bt_wc_email_from_address', 9998, 4 ); */


/**
 * Hook into "from" name (for any woocommerce email) to:
 *
 * - if sellers, set from to the seller's name along with marketplace name (blogname)
 *
 * @param $from_name string
 * @param $wc_email \WC_Email
 *
 * @return string
 */
function bt_wc_email_from_name( $from_name, $wc_email ) {

	// things to do if we are using sellers
	if ( bt_use_sellers() ) {

		if ( in_array( $wc_email->id, bt_wc_emails_from_seller() ) ) {
			$seller_data = bt_get_order_seller_data( $wc_email->object->get_id() );
			//error_log( 'WOO EMAIL FROM NAME FN. FILTERING email = ' . $wc_email->id );

			// get seller email and change the recipient to that so long as it exists (it always should)
			if ( bt_use_sellers( 'email_from_use_marketplace_only' ) ) {
				// a hacky override for DM Delivery site which doesn't need both seller and site name
				$seller_name = get_bloginfo( 'name' );
			} else {
				$seller_name = $seller_data->post->post_title . ' via ' . get_bloginfo( 'name' );
			}

			if ( ! empty( $seller_name ) ) {
				$from_name = $seller_name;
			}
		}

	}

	//error_log( 'WOO EMAIL [' . $wc_email->id . '] FROM NAME FN. from_name result = ' . $from_name );

	return $from_name;
}
add_filter( 'woocommerce_email_from_name', 'bt_wc_email_from_name', 9998, 2 );


/**
 * Hook into any email sent TO a seller, so as to:
 *
 * - make it go to the seller's email address.
 *   - this replaces the WC setting for the email.
 *   - admin email for site will get BCC via bt_wc_email_headers().
 *
 * @param string $recipient email address
 * @param object $object could be WC_Order, some form of WC_Product, or whatever else woo objects can be for emails
 *
 * @return string
 */
function bt_wc_email_to_seller( $recipient, $object ) {
	//error_log( 'EMAIL TO SELLER FILTER. recipient = ' . $recipient . ' / object type = ' . $object->get_type() );

	// prevent running on woo email settings screen
	$page = $_GET['page'] = isset( $_GET['page'] ) ? $_GET['page'] : '';
	if ( 'wc-settings' === $page ) {
		return $recipient;
	}

	// things to do if we are using sellers
	if ( bt_use_sellers() ) {

		// find seller based on object being an order or product
		if ( $object->get_type() == 'shop_order' ) {
			$seller_email = bt_get_order_seller_email( $object->get_id() );
		} else {
			// some kind of product
			$seller_email = get_the_author_meta( 'user_email', bt_get_author_id_for_post( $object->get_id() ) );
		}

		// change the recipient to that so long as it exists (it always should)
		//error_log( 'WOO EMAIL RECIPIENT FN. recip = ' . $recipient . ' seller = ' . $seller_email );
		if ( ! empty( $seller_email ) ) {
			$recipient = $seller_email;
		}
	}

	//error_log( 'WOO EMAIL RECIPIENT FN. recipient result = ' . $recipient );

	return $recipient;
}
foreach ( bt_wc_emails_to_seller() as $email_id ) {
	add_filter( 'woocommerce_email_recipient_' . $email_id, 'bt_wc_email_to_seller', 9998, 2 );
}


/**
 * Filter args to woocommerce email order items, if bt-sellers or bt-ecomm option exists:
 * - show image
 * - larger image dimensions
 */
add_filter( 'woocommerce_email_order_items_args', function( $args ) {
	//error_log( 'woocommerce_email_order_items_args = ' . print_r( $args, true ) );

	if (
		bt_use_sellers( 'email_show_product_images' )
		|| bt_ecomm_option( 'email_show_product_images' )
	) {
		$args['show_image'] = true;

    if ( ! $dims = bt_ecomm_option( 'email_product_image_dimensions' ) ) {
      $dims = [64, 64];
    }
		$args['image_size'] = $dims;
	}

	return $args;
}, 9988 );


/**
 * Filter recipient of completed order email:
 * - if all items are virtual, blank out recipient so that email won't send.
 */
add_filter( 'woocommerce_email_recipient_customer_completed_order', function ( $recipient, $order ) {
  $page = $_GET['page'] = $_GET['page'] ?? '';
  if ( 'wc-settings' === $page ) {
    // don't do anything within WC settings, so it all behaves normally
    return $recipient;
  }

  if ( bt_ecomm_option( 'skip_completed_email_when_virtual_order' ) ) {
    $order_is_all_virtual = true;
    foreach ( $order->get_items() as $item ) {
      $is_virtual = get_post_meta( $item->get_product_id(), '_virtual', true );
      //error_log( 'ITEM ID [' . $item->get_product_id() . '] IS VIRTUAL = ' . $is_virtual );
      if ( $is_virtual != 'yes' ) {
        $order_is_all_virtual = false;
        break;
      }
    }
    if ( $order_is_all_virtual ) {
      //error_log( 'SKIPPING ORDER COMPLETED EMAIL - ALL ITEMS ARE VIRTUAL!' );
      $recipient = '';
    }
  }

  return $recipient;
}, 10, 2 );


/**
 * Filter WC email footer text to:
 * - append custom HTML content if it exists, and if so, wrap existing text in a <p> (contents will go through
 *    wp_autop() and we are adding more paragraphs)
 */
add_filter( 'woocommerce_email_footer_text', function( $content ) {
  if ( $additional_content = bt_ecomm_option( 'woocommerce_email_footer_text_additional' ) ) {

    if ( ! empty( $content ) ) {
      // wrap existing content in <p> since we're adding more <p>s
      $content = '<p>' . $content . '</p>';
    }

    $content .= $additional_content;
  }

  return $content;
}, 9987 );


/**
 * Send a "Customer New Account" email to the seller, which will:
 * - reset password and send the new password in the email
 * - include custom content via email template override, if exists
 * - record that seller received (was sent) the welcome email
 *
 * Emails only actually send in production env; else a log entry is made.
 *
 * @param int $user ID of the user account
 */
function bt_send_seller_welcome_email( $user ) {
	// disable password changed email from WP core, so that doesn't get sent in addition to the welcome email
	add_filter( 'send_password_change_email', function ( $send, $user, $userdata ) {
		return false;
	}, 9988, 3 );

	// set heading and subject (these may match defaults but would be blank if we don't set them)
	$heading = 'Welcome to ' . get_option( 'blogname' );
	$subject = 'Welcome to ' . get_option( 'blogname' );

	// Get WooCommerce email objects
	$mailer = WC()->mailer()->get_emails();

	// assign heading & subject to chosen object (will be blank otherwise)
	$mailer['WC_Email_Customer_New_Account']->heading = $heading;
	$mailer['WC_Email_Customer_New_Account']->settings['heading'] = $heading;
	$mailer['WC_Email_Customer_New_Account']->subject = $subject;
	$mailer['WC_Email_Customer_New_Account']->settings['subject'] = $subject;

	//error_log( __METHOD__ . ': email = ' . print_r( $mailer['WC_Email_Customer_New_Account'], 1 ) );

	// reset password to facilitate access and helping lots of sellers
	$new_pass = wp_generate_password();
	wp_update_user( [
		'ID' => $user,
		'user_pass' => $new_pass,
	] );

	// send the email
	if ( defined( 'BT_ENV') && BT_ENV == 'dev' ) {
		// just log if dev env
		error_log( 'SEND NEW ACCOUNT EMAIL: ' . $heading . ' / user = ' . $user .' / pass = ' . $new_pass );
	} else {
		$mailer['WC_Email_Customer_New_Account']->trigger( $user, $new_pass, true );
		error_log( 'SENT NEW ACCOUNT EMAIL: ' . $heading . ' / user = ' . $user );
	}

	// record that the welcome email has been sent
	update_user_meta( $user, 'received_welcome_email', '1' );
}


/**
 * Add content to emails before order details:
 * - time window
 * - gift recipient info, if it's a gift
 */
add_action( 'woocommerce_email_order_details', function( $order, $sent_to_admin, $plain_text, $email ) {
	if ( $time_windows = bt_ecomm_option( 'checkout_time_window_options' ) ) {
		if ( ! $time_window_label = bt_ecomm_option( 'checkout_time_window_label' ) ) {
			$time_window_label = 'Time Window';
		}

		if ( $selected_window = get_post_meta( $order->get_id(), 'time_window', true ) ) {
			echo '<p>' . $time_window_label . ': <strong>' . $selected_window . '</strong></p>';
		}
	}

	if ( $is_a_gift = bt_ecomm_option( 'checkout_is_a_gift' ) ) {
		echo bt_order_gift_recipient_html( $order->get_id() );
	}
}, 9, 4 );


/**
 * AJAX action to trigger a password reset and sending of welcome email to a seller user.
 */
add_action( 'wp_ajax_bt_send_welcome_email', 'bt_send_welcome_email' );
//add_action( 'wp_ajax_nopriv_bt_send_welcome_email', 'bt_send_welcome_email' );
function bt_send_welcome_email() {
	if ( bt_can_admin_marketplace() ) {
		if ( isset( $_REQUEST['user_id'] ) && is_numeric( $_REQUEST['user_id'] ) ) {
			bt_send_seller_welcome_email( $_REQUEST['user_id'] );
			echo 'Success!';
		} else {
			echo 'No or invalid user ID provided.';
		}
	} else {
		echo 'Failed. You must be an administrator.';
	}
	wp_die();
}
